@@ -1,5 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2008, 2010 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2009 Aurélien Ansel <camumus@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -237,7 +238,6 @@ def add_item_to_item_set(item_set, item):
 def get_first_table(grammar):
     symbols = grammar.symbols
     rules = grammar.rules
-    lexical_table = grammar.tokenizer.lexical_table
 
     first = {}
     # Initialize
@@ -356,7 +356,6 @@ def find_handles(grammar, item_set):
 
     handles = set()
     shift = False
-    new_item_set = set()
     for name, i, j, look_ahead in item_set:
         if j == len(rules[name][i]):
             handles.add((name, i, look_ahead))
@@ -524,11 +523,8 @@ def build_graph(grammar, map, token_table, symbol_table):
     lines.append('}\n')
 
     # Write the file
-    file = open('/tmp/graph.dot', 'w')
-    try:
+    with open('/tmp/graph.dot', 'w') as file:
         file.write(''.join(lines))
-    finally:
-        file.close()
 
 
 
@@ -551,7 +547,6 @@ def pprint_stack(stack, active_nodes, data, file=None):
         for last_node in last_nodes:
             in_paths.append([last_node] + path)
 
-    lines = []
     for path in paths:
         line = []
         for node_idx in path:
