@@ -1,8 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2003-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2007 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2003-2010 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2007, 2010 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2008 Sylvain Taverne <taverne.sylvain@gmail.com>
 # Copyright (C) 2008 Wynand Winterbach <wynand.winterbach@gmail.com>
+# Copyright (C) 2008, 2010 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2009 Aurélien Ansel <camumus@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -202,7 +204,7 @@ def decode_target(target):
 
 # Escape and unescape strings
 def escape(s):
-    return s.replace('\n', '\\n').replace('"', '\\"')
+    return s.replace('\r', '\\r').replace('\n', '\\n').replace('"', '\\"')
 
 
 expr = compile(r'(\\.)')
@@ -536,6 +538,7 @@ class POFile(TextFile):
         messages = self.messages
         if key in messages:
             unit = messages[key]
+            unit.target = target
         else:
             unit = POUnit(comments, context, source, target, {}, fuzzy)
             messages[key] = unit
@@ -572,6 +575,10 @@ class POFile(TextFile):
         return None
 
 
+    def set_msgstr(self, source, target, context=None):
+        self._set_message(context, [source], [target])
+
+
     def gettext(self, source, context=None):
         """Returns the translation of the given message id.
 
