@@ -1,5 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2005-2007, 2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2011 Hervé Cauwelier <herve@oursours.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@ def register_handler_class(handler_class):
         handler_classes[mimetype] = handler_class
 
 
-def get_handler_class_by_mimetype(mimetype):
+def get_handler_class_by_mimetype(mimetype, soft=False):
     if mimetype is not None:
         if mimetype in handler_classes:
             return handler_classes[mimetype]
@@ -32,4 +33,7 @@ def get_handler_class_by_mimetype(mimetype):
         if main_type in handler_classes:
             return handler_classes[main_type]
 
-    raise ValueError
+    if soft:
+        return None
+
+    raise ValueError, mimetype
