@@ -1,7 +1,9 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2003-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2006 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2007 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2003-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2006 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2007 Sylvain Taverne <taverne.sylvain@gmail.com>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2008, 2010 Henry Obein <henry.obein@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,19 +23,17 @@ from itools.datatypes import XMLContent, XMLAttribute
 from itools.handlers import register_handler_class
 from itools.xml import START_ELEMENT, END_ELEMENT, TEXT
 from itools.xml import stream_to_str, get_qname, get_attribute_qname
-from itools.xml import get_end_tag, get_doctype, get_element
+from itools.xml import get_doctype, get_element, get_namespace
 from itools.xmlfile import XMLFile
 
-
-xhtml_uri = 'http://www.w3.org/1999/xhtml'
-
+# Import from here
+from schema import html_namespace, html_uri as xhtml_uri
 
 
 
 def get_start_tag(value):
     tag_uri, tag_name, attributes = value
-    qname = get_qname(tag_uri, tag_name)
-    s = '<%s' % qname
+    s = '<%s' % get_qname(tag_uri, tag_name)
     # Output the attributes
     for attr_uri, attr_name in attributes:
         value = attributes[(attr_uri, attr_name)]
@@ -43,11 +43,24 @@ def get_start_tag(value):
     return s + '>'
 
 
+def get_end_tag(value):
+    tag_uri, tag_name = value
+    if tag_uri and tag_uri != xhtml_uri:
+        raise ValueError, tag_uri
+
+    # Case 1: empty
+    schema = html_namespace.get_element_schema(tag_name)
+    if getattr(schema, 'is_empty', False):
+        return ''
+    # Case 2: not empty
+    return '</%s>' % tag_name
+
+
 stream_to_html_map = (
     lambda x: '',                      # XML_DECL
     lambda x: get_doctype(x[0], x[1]), # DOCUMENT_TYPE
     get_start_tag,                     # START_ELEMENT
-    lambda x: get_end_tag(x[0], x[1]), # END_ELEMENT
+    get_end_tag,                       # END_ELEMENT
     XMLContent.encode,                 # TEXT
     lambda x: '<!--%s-->' % x,         # COMMENT
     lambda x: '',                      # PI
@@ -106,6 +119,21 @@ def stream_to_str_as_html(stream, encoding='UTF-8'):
     return stream_to_html(stream, encoding)
 
 
+def stream_is_empty(stream):
+
+    for type, value, line in stream:
+        if type == TEXT:
+            if value.replace('&nbsp;', '').strip():
+                return False
+        elif type == START_ELEMENT:
+            tag_uri, tag_name, attributes = value
+            if tag_name in ('img', 'object'):
+                # If the document contains at leat one image
+                # or one object (i.e. flash object) it is not empty
+                return False
+    return True
+
+
 ###########################################################################
 # Document
 ###########################################################################
@@ -180,8 +208,12 @@ class XHTMLFile(XMLFile):
     def set_body(self, events):
         body = self.get_body()
         events = self.events[:body.start+1] + events + self.events[body.end:]
+        if stream_to_str(events) == stream_to_str(self.events):
+            return False
+
         self.set_changed()
         self.events = events
+        return True
 
 
     def is_empty(self):
@@ -192,17 +224,7 @@ class XHTMLFile(XMLFile):
         else:
             events = body.events
 
-        for type, value, line in events:
-            if type == TEXT:
-                if value.replace('&nbsp;', '').strip():
-                    return False
-            elif type == START_ELEMENT:
-                tag_uri, tag_name, attributes = value
-                if tag_name in ('img', 'object'):
-                    # If the document contains at leat one image
-                    # or one object (i.e. flash object) it is not empty
-                    return False
-        return True
+        return stream_is_empty(events)
 
 
 
