@@ -18,24 +18,22 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.core import add_type
 from handlers import SetupConf
-from metadata import PKGINFOFile
-from packages import packages_infos
-from utils import get_compile_flags, get_files, get_manifest, make_version
+from utils import get_compile_flags, get_manifest
 from utils import setup, get_config, OptionalExtension
 
 
 __all__ = [
     'get_compile_flags',
     'get_config',
-    'get_files',
     'get_manifest',
-    'make_version',
-    'packages_infos',
     'OptionalExtension',
-    'PKGINFOFile',
     'setup',
     'SetupConf']
 
-add_type('text/x-egg-info', '.egg-info')
+# Git: optional
+try:
+    from git import open_worktree
+    __all__.append('open_worktree')
+except ImportError:
+    pass
