@@ -22,74 +22,16 @@ from distutils import core
 from distutils.core import Extension
 from distutils.command.build_ext import build_ext
 from distutils.errors import LinkError
-from os import listdir
-from os.path import exists, isdir, join as join_path
-from re import compile
+from os.path import exists, join as join_path
 from sys import _getframe, argv
 
 # Import from itools
 from itools.core import freeze, get_pipe, get_version
-from itools.git import open_worktree
 from itools.handlers import ro_database
 from handlers import SetupConf
 
 
 
-def make_version(cwd='.'):
-    """This function finds out the version number from the source, this will
-    be written to the 'version.txt' file, which will be read once the software
-    is installed to get the version number.
-    """
-    worktree = open_worktree(cwd)
-    # The name of the active branch
-    branch = worktree.get_branch_name()
-    if branch is None:
-        return None
-
-    # The tag
-    description = worktree.git_describe()
-
-    # The version name
-    if description:
-        tag, n, commit = description
-        if tag.startswith(branch):
-            version = tag
-        else:
-            version = '%s-%s' % (branch, tag)
-        # Exact match
-        if n == 0:
-            return version
-    else:
-        version = branch
-
-    # Get the timestamp
-    head = worktree.get_metadata()
-    timestamp = head['committer_date']
-    timestamp = timestamp.strftime('%Y%m%d%H%M')
-    return '%s-%s' % (version, timestamp)
-
-
-
-def get_files(excluded_paths, filter=lambda x: True):
-    for name in listdir('.'):
-        if name in excluded_paths:
-            continue
-
-        if isdir(name):
-            stack = [name]
-            while stack:
-                base = stack.pop()
-                for name in listdir(base):
-                    path = join_path(base, name)
-                    if isdir(path):
-                        stack.append(path)
-                    elif filter(name):
-                        yield path
-        elif filter(name):
-            yield name
-
-
-
 class OptionalExtension(Extension):
     """An Optional Extension is a C extension that complements the package
     without being mandatory. It typically depends on external libraries. If the
@@ -157,18 +99,11 @@ def get_config():
 
 
 def get_manifest():
-    worktree = open_worktree('.', soft=True)
-    if worktree:
-        exclude = frozenset(['.gitignore'])
-        return [ x for x in worktree.get_filenames() if x not in exclude ]
+    from git import open_worktree
 
-    # No git: find out source files
-    config = get_config()
-    target_languages = config.get_value('target_languages')
-
-    exclude = frozenset(['.git', 'build', 'dist'])
-    bad_files = compile('.*(~|pyc|%s)$' % '|'.join(target_languages))
-    return get_files(exclude, filter=lambda x: not bad_files.match(x))
+    worktree = open_worktree('.')
+    exclude = frozenset(['.gitignore'])
+    return [ x for x in worktree.get_filenames() if x not in exclude ]
 
 
 
@@ -242,7 +177,7 @@ def setup(ext_modules=freeze([])):
                packages = packages,
                package_data = package_data,
                # Requires / Provides
-               requires = config.get_value('requires'),
+               requires = list(config.get_value('requires')),
                provides = config.get_value('provides'),
                # Scripts
                scripts = scripts,
