@@ -1,8 +1,9 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2003-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2006 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2007-2008 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2007-2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2003-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2006, 2010 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2007-2008, 2010 Sylvain Taverne <taverne.sylvain@gmail.com>
+# Copyright (C) 2007-2011 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,17 +26,18 @@ language I could imagine.
 # Import from the Standard Library
 from functools import partial
 from re import compile
-from types import GeneratorType
+from types import GeneratorType, MethodType
 
 # Import from itools
-from itools.core import freeze
+from itools.core import freeze, prototype, is_prototype
 from itools.datatypes import Boolean
 from itools.gettext import MSG
+from itools.log import log_error
 from itools.uri import Path, Reference, get_reference
-from itools.xml import XMLParser, find_end, stream_to_str
+from itools.xml import XMLParser, find_end, get_attr_datatype, stream_to_str
 from itools.xml import DOCUMENT_TYPE, START_ELEMENT, END_ELEMENT, TEXT
 from itools.xml import xmlns_uri
-from itools.xml import get_attr_datatype
+from itools.xml import is_xml_stream
 from itools.xmlfile import XMLFile, get_units
 from itools.html import xhtml_uri
 from itools.html import stream_to_str_as_html, stream_to_str_as_xhtml
@@ -50,8 +52,6 @@ class STLError(StandardError):
     pass
 
 
-ERR_EXPR_NAME = 'name "%s" not found in "${%s}" expression'
-ERR_EXPR_VALUE = 'unexpected value of type "%s" in "${%s}" expression'
 ERR_EXPR_XML = 'expected XML stream not "%s" in "${%s}" expression'
 
 
@@ -82,29 +82,17 @@ def evaluate(expression, stack, repeat_stack):
         path = path[1:]
 
     # Traverse
-    value = stack.lookup(path[0])
+    err = "evaluation of '%s' failed, '%s' could not be resolved"
+    try:
+        value = stack.lookup(path[0])
+    except STLError:
+        raise STLError, err % (expression, path[0])
+
     for name in path[1:]:
         try:
-            value = value[name]
-        except KeyError:
-            raise STLError, ERR_EXPR_NAME % (name, expression)
-
-
-    # Call
-    if hasattr(value, '__call__'):
-        try:
-            value = value()
-        except AttributeError, error_value:
-            # XXX "callable" could return true even if the object is not
-            # callable (see Python's documentation).
-            #
-            # This happens, for example, in the context of Zope, maybe
-            # because of extension classes and acquisition. So we catch
-            # the AttributeError exception, we should test also for the
-            # exception value to be "__call__". This is dangereous
-            # because we could hide real errors. Further exploration
-            # needed..
-            pass
+            value = lookup(value, name)
+        except STLError:
+            raise STLError, err % (expression, name)
 
     return value
 
@@ -127,18 +115,35 @@ def evaluate_repeat(expression, stack, repeat_stack):
 ###########################################################################
 # Namespace
 ###########################################################################
+
+def lookup(namespace, name):
+    # Case 1: dict
+    if type(namespace) is dict:
+        if name not in namespace:
+            raise STLError
+        return namespace[name]
+
+    # Case 2: instance
+    try:
+        value = getattr(namespace, name)
+    except AttributeError:
+        log_error('lookup failed', domain='itools.stl')
+        raise STLError
+    if type(value) is MethodType:
+        value = value()
+    return value
+
+
 class NamespaceStack(list):
     """This class represents a namespace stack as used by STL. A variable
     is looked up in the stack from the top to the bottom until found.
     """
 
     def lookup(self, name):
-        stack = self[:]
-        stack.reverse()
-        for namespace in stack:
+        for namespace in reversed(self):
             try:
-                return namespace[name]
-            except KeyError:
+                return lookup(namespace, name)
+            except STLError:
                 pass
 
         raise STLError, 'name "%s" not found in the namespace' % name
@@ -157,7 +162,7 @@ subs_expr = compile("\$\{(.+?)\}")
 
 
 def substitute_boolean(data, stack, repeat_stack, encoding='utf-8'):
-    if isinstance(data, bool):
+    if type(data) is bool:
         return data
 
     match = subs_expr_solo.match(data)
@@ -175,7 +180,7 @@ def substitute_attribute(data, stack, repeat_stack, encoding='utf-8'):
     Returns a tuple with the interpreted string and the number of
     substitutions done.
     """
-    if not isinstance(data, str):
+    if type(data) is not str:
         raise ValueError, 'byte string expected, not %s' % type(data)
     # Solo, preserve the value None
     match = subs_expr_solo.match(data)
@@ -186,7 +191,7 @@ def substitute_attribute(data, stack, repeat_stack, encoding='utf-8'):
         if value is None:
             return None, 1
         # Send the string
-        if isinstance(value, MSG):
+        if is_prototype(value, MSG):
             return value.gettext().encode(encoding), 1
         elif type(value) is unicode:
             return value.encode(encoding), 1
@@ -199,7 +204,7 @@ def substitute_attribute(data, stack, repeat_stack, encoding='utf-8'):
         if value is None:
             return ''
         # Send the string
-        if isinstance(value, MSG):
+        if is_prototype(value, MSG):
             return value.gettext().encode(encoding)
         elif type(value) is unicode:
             return value.encode(encoding)
@@ -223,23 +228,26 @@ def substitute(data, stack, repeat_stack, encoding='utf-8'):
         if i % 2:
             # Evaluate expression
             value = evaluate(segment, stack, repeat_stack)
-            # An STL template
-            if isinstance(value, STLTemplate):
-                value = value.render()
+            # An STL template (duck typing)
+            render = getattr(value, 'render', None)
+            if render:
+                value = render()
             # Ignore if None
             if value is None:
                 continue
-            # Yield
-            if isinstance(value, MSG):
+
+            # Case MSG: it returns <unicode> or <XMLParser>
+            if is_prototype(value, MSG):
                 value = value.gettext()
+
+            # Yield
+            if type(value) is unicode:
                 yield TEXT, value.encode(encoding), 0
-            elif isinstance(value, (list, GeneratorType, XMLParser)):
+            elif is_xml_stream(value):
                 for x in value:
                     if type(x) is not tuple:
                         raise STLError, ERR_EXPR_XML % (type(x), segment)
                     yield x
-            elif type(value) is unicode:
-                yield TEXT, value.encode(encoding), 0
             else:
                 yield TEXT, str(value), 0
         elif segment:
@@ -258,7 +266,7 @@ def stl(document=None, namespace=freeze({}), prefix=None, events=None,
     if prefix is not None:
         stream = set_prefix(events, prefix)
         events = list(stream)
-    elif isinstance(events, (GeneratorType, XMLParser)):
+    elif type(events) in (GeneratorType, XMLParser):
         events = list(events)
 
     # Initialize the namespace stacks
@@ -329,13 +337,13 @@ def process_start_tag(tag_uri, tag_name, attributes, stack, repeat, encoding):
     return START_ELEMENT, (tag_uri, tag_name, aux), None
 
 
-def process(events, start, end, stack, repeat_stack, encoding, skip_events):
+def process(events, start, end, stack, re_stack, encoding, skip_events):
     skip = set()
     i = start
     while i < end:
         event, value, line = events[i]
         if event == TEXT:
-            stream = substitute(value, stack, repeat_stack, encoding)
+            stream = substitute(value, stack, re_stack, encoding)
             for event, value, kk in stream:
                 yield event, value, line
         elif event == START_ELEMENT:
@@ -343,47 +351,45 @@ def process(events, start, end, stack, repeat_stack, encoding, skip_events):
             # stl:repeat
             if stl_repeat in attributes:
                 attributes = attributes.copy()
-                expr = attributes.pop(stl_repeat)
-                name, values = evaluate_repeat(expr, stack, repeat_stack)
-                # Build new namespace stacks
-                loops = []
+                re_expr = attributes.pop(stl_repeat)
+                if_expr = attributes.pop(stl_if, None)
+                loop_end = find_end(events, i)
+                i += 1
+                # repeat...
+                name, values = evaluate_repeat(re_expr, stack, re_stack)
                 n_values = len(values)
                 for j, value in enumerate(values):
-                    loop_stack = stack[:]
-                    loop_stack.append({name: value})
-                    loop_repeat = repeat_stack[:]
-                    loop_repeat.append(
+                    # 1. New stacks
+                    stack.append({name: value})
+                    re_stack.append(
                         {name: {'index': j,
                                 'start': j == 0,
                                 'end': j == n_values - 1,
                                 'even': j % 2 and 'odd' or 'even'}})
-                    loops.append((loop_stack, loop_repeat))
-                # Filter the branches when "stl:if" is present
-                if stl_if in attributes:
-                    expr = attributes.pop(stl_if)
-                    loops = [ (x, y) for x, y in loops
-                              if evaluate_if(expr, x, y) ]
-                # Process the loops
-                loop_end = find_end(events, i)
-                i += 1
-                for loop_stack, loop_repeat in loops:
+                    # 2. stl:if
+                    if if_expr and not evaluate_if(if_expr, stack, re_stack):
+                        continue
+                    # 3. Process
                     x = process_start_tag(tag_uri, tag_name, attributes,
-                                          loop_stack, loop_repeat, encoding)
+                                          stack, re_stack, encoding)
                     if x is not None:
                         yield x
-                    for y in process(events, i, loop_end, loop_stack,
-                                     loop_repeat, encoding, skip_events):
+                    for y in process(events, i, loop_end, stack, re_stack,
+                                     encoding, skip_events):
                         yield y
                     if x is not None:
                         yield events[loop_end]
+                    # 4. Restore stacks
+                    stack.pop()
+                    re_stack.pop()
                 i = loop_end
             # stl:if
             elif stl_if in attributes:
                 attributes = attributes.copy()
                 expression = attributes.pop(stl_if)
-                if evaluate_if(expression, stack, repeat_stack):
+                if evaluate_if(expression, stack, re_stack):
                     x = process_start_tag(tag_uri, tag_name, attributes,
-                                          stack, repeat_stack, encoding)
+                                          stack, re_stack, encoding)
                     if x is None:
                         skip.add(find_end(events, i))
                     else:
@@ -394,7 +400,7 @@ def process(events, start, end, stack, repeat_stack, encoding, skip_events):
             else:
                 if tag_uri != stl_uri:
                     x = process_start_tag(tag_uri, tag_name, attributes,
-                                          stack, repeat_stack, encoding)
+                                          stack, re_stack, encoding)
                     if x is None:
                         skip.add(find_end(events, i))
                     else:
@@ -415,28 +421,44 @@ def process(events, start, end, stack, repeat_stack, encoding, skip_events):
 ########################################################################
 css_uri_expr = compile (r"url\(([a-zA-Z0-9\./%\-\_]*/%3[bB]{1}download)\);")
 
-def set_prefix(stream, prefix, ns_uri=xhtml_uri):
+def set_prefix(stream, prefix, ns_uri=xhtml_uri, uri=None):
     if isinstance(prefix, str):
         prefix = Path(prefix)
-    rewrite = partial(resolve_pointer, prefix)
+
+    ref = None
+    if uri:
+        ref = Reference(scheme=uri.scheme, authority=uri.authority,
+                        path='/', query={})
+    rewrite = partial(resolve_pointer, prefix, ref)
 
     return rewrite_uris(stream, rewrite, ns_uri)
 
 
 
-def resolve_pointer(offset, value):
+def resolve_pointer(offset, reference, value):
     # FIXME Exception for STL
     if value[:2] == '${':
         return value
 
     # Absolute URI or path
     uri = get_reference(value)
-    if uri.scheme or uri.authority or uri.path.is_absolute():
+    if uri.scheme or uri.authority:
         return value
 
     # Resolve Path
-    path = offset.resolve(uri.path)
-    value = Reference('', '', path, uri.query.copy(), uri.fragment)
+    if uri.path.is_absolute():
+        if reference is None:
+            return value
+        # Do not call resolve with absolute path
+        path = uri.path
+    else:
+        path = offset.resolve(uri.path)
+
+    scheme = authority = ''
+    if reference:
+        scheme = reference.scheme
+        authority = reference.authority
+    value = Reference(scheme, authority, path, uri.query.copy(), uri.fragment)
     return str(value)
 
 
@@ -470,6 +492,12 @@ def rewrite_uris(stream, rewrite, ns_uri=xhtml_uri):
                             else:
                                 chunks.append(segment)
                         value = ''.join(chunks)
+                    # <object type="application/x-shockwave-flash" data="...">
+                    elif tag_name == 'object':
+                        if attr_name == 'data':
+                            attr_type = attributes.get((attr_uri, 'type'))
+                            if attr_type == 'application/x-shockwave-flash':
+                                value = rewrite(value)
                     # <param name="movie" value="X" />
                     elif tag_name == 'param':
                         if attr_name == 'value':
@@ -500,31 +528,29 @@ class STLFile(XMLFile):
 # Templates
 ###########################################################################
 
-class STLTemplate(object):
+class STLTemplate(prototype):
 
-    def __init__(self):
-        """To be overriden.  It should accept and keep the information
-        required by 'get_template' and 'get_namespace'.
-        """
-        raise NotImplementedError
-
-
-    def show(self):
-        return True
+    template = None
+    show = True
 
 
     def get_template(self):
-        raise NotImplementedError
+        if type(self.template) is list:
+            return self.template
+
+        error = 'template variable of unexpected type "%s"'
+        raise TypeError, error % type(self.template).__name__
 
 
-    def get_namespace(self):
-        return {}
-
-
-    def render(self):
-        if self.show() is False:
+    def render(self, mode='events'):
+        if not self.show:
             return None
 
+        # Case 1: a ready made list of events
         template = self.get_template()
-        namespace = self.get_namespace()
-        return stl(template, namespace)
+        if type(template) is list:
+            return stl(events=template, namespace=self, mode=mode)
+
+        # Case 2: we assume it is a handler
+        return stl(template, self, mode=mode)
+
